package org.eclipse.incquery.uml.derivedfeatures;

import org.eclipse.incquery.runtime.api.IncQueryEngine;
import org.eclipse.incquery.runtime.api.impl.BaseGeneratedPatternGroup;
import org.eclipse.incquery.runtime.exception.IncQueryException;
import org.eclipse.incquery.uml.derivedfeatures.ActionInputMatcher;
import org.eclipse.incquery.uml.derivedfeatures.ActionOutputMatcher;
import org.eclipse.incquery.uml.derivedfeatures.ActivityEdgeInGroupMatcher;
import org.eclipse.incquery.uml.derivedfeatures.ActivityGroupContainedEdgeMatcher;
import org.eclipse.incquery.uml.derivedfeatures.ActivityGroupContainedNodeMatcher;
import org.eclipse.incquery.uml.derivedfeatures.ActivityGroupMatcher;
import org.eclipse.incquery.uml.derivedfeatures.ActivityGroupSubgroupMatcher;
import org.eclipse.incquery.uml.derivedfeatures.ActivityGroupSuperGroupMatcher;
import org.eclipse.incquery.uml.derivedfeatures.ActivityNodeInGroupMatcher;
import org.eclipse.incquery.uml.derivedfeatures.ActivityNodeMatcher;
import org.eclipse.incquery.uml.derivedfeatures.AssociationEndTypeMatcher;
import org.eclipse.incquery.uml.derivedfeatures.ClassExtensionMatcher;
import org.eclipse.incquery.uml.derivedfeatures.ClassSuperClassMatcher;
import org.eclipse.incquery.uml.derivedfeatures.ClassifierAllParentsMatcher;
import org.eclipse.incquery.uml.derivedfeatures.ClassifierAttributeMatcher;
import org.eclipse.incquery.uml.derivedfeatures.ClassifierFeatureMatcher;
import org.eclipse.incquery.uml.derivedfeatures.ClassifierGeneralMatcher;
import org.eclipse.incquery.uml.derivedfeatures.ClassifierParentsMatcher;
import org.eclipse.incquery.uml.derivedfeatures.ConnectableElementEndMatcher;
import org.eclipse.incquery.uml.derivedfeatures.ConnectorKindMatcher;
import org.eclipse.incquery.uml.derivedfeatures.DeploymentTargetDeployedElementMatcher;
import org.eclipse.incquery.uml.derivedfeatures.DirectedRelationshipSourceMatcher;
import org.eclipse.incquery.uml.derivedfeatures.DirectedRelationshipTargetMatcher;
import org.eclipse.incquery.uml.derivedfeatures.ElementOwnedElementMatcher;
import org.eclipse.incquery.uml.derivedfeatures.ElementOwnerMatcher;
import org.eclipse.incquery.uml.derivedfeatures.EncapsulatedClassifierOwnedPortMatcher;
import org.eclipse.incquery.uml.derivedfeatures.ExtensionMetaclassEndMatcher;
import org.eclipse.incquery.uml.derivedfeatures.ExtensionMetaclassMatcher;
import org.eclipse.incquery.uml.derivedfeatures.FeatureFeaturingClassifierMatcher;
import org.eclipse.incquery.uml.derivedfeatures.MessageMessageKindMatcher;
import org.eclipse.incquery.uml.derivedfeatures.NamedElementClientDependencyMatcher;
import org.eclipse.incquery.uml.derivedfeatures.NamedElementNamespaceMatcher;
import org.eclipse.incquery.uml.derivedfeatures.NamedElementQualifiedNameMatcher;
import org.eclipse.incquery.uml.derivedfeatures.NamespaceImportedMemberMatcher;
import org.eclipse.incquery.uml.derivedfeatures.NamespaceMemberMatcher;
import org.eclipse.incquery.uml.derivedfeatures.NamespaceOwnedMemberMatcher;
import org.eclipse.incquery.uml.derivedfeatures.OpaqueExpressionResultMatcher;
import org.eclipse.incquery.uml.derivedfeatures.OperationReturnResultMatcher;
import org.eclipse.incquery.uml.derivedfeatures.PackageNestedPackageMatcher;
import org.eclipse.incquery.uml.derivedfeatures.PackageNestingPackageMatcher;
import org.eclipse.incquery.uml.derivedfeatures.PackageOwnedStereotypeMatcher;
import org.eclipse.incquery.uml.derivedfeatures.PackageOwnedTypeMatcher;
import org.eclipse.incquery.uml.derivedfeatures.PackageVisibleMemberMatcher;
import org.eclipse.incquery.uml.derivedfeatures.PropertyIsCompositeMatcher;
import org.eclipse.incquery.uml.derivedfeatures.ProtocolTransitionReferredMatcher;
import org.eclipse.incquery.uml.derivedfeatures.RedefinableElementRedefinedElementMatcher;
import org.eclipse.incquery.uml.derivedfeatures.RedefinableElementRedefinitionContextMatcher;
import org.eclipse.incquery.uml.derivedfeatures.RedefinableTemplateSignatureInheritedParameterMatcher;
import org.eclipse.incquery.uml.derivedfeatures.RelationshipRelatedElementMatcher;
import org.eclipse.incquery.uml.derivedfeatures.StateIsCompositeMatcher;
import org.eclipse.incquery.uml.derivedfeatures.StateIsComposite_0Matcher;
import org.eclipse.incquery.uml.derivedfeatures.StateIsOrthogonalMatcher;
import org.eclipse.incquery.uml.derivedfeatures.StructuredClassifierPartMatcher;
import org.eclipse.incquery.uml.derivedfeatures.StructuredClassifierRoleMatcher;
import org.eclipse.incquery.uml.derivedfeatures.TypePackageMatcher;
import org.eclipse.incquery.uml.derivedfeatures.VertexIncomingMatcher;
import org.eclipse.incquery.uml.derivedfeatures.VertexOutgoingMatcher;
import org.eclipse.incquery.uml.derivedfeatures.util.ActionInputQuerySpecification;
import org.eclipse.incquery.uml.derivedfeatures.util.ActionOutputQuerySpecification;
import org.eclipse.incquery.uml.derivedfeatures.util.ActivityEdgeInGroupQuerySpecification;
import org.eclipse.incquery.uml.derivedfeatures.util.ActivityGroupContainedEdgeQuerySpecification;
import org.eclipse.incquery.uml.derivedfeatures.util.ActivityGroupContainedNodeQuerySpecification;
import org.eclipse.incquery.uml.derivedfeatures.util.ActivityGroupQuerySpecification;
import org.eclipse.incquery.uml.derivedfeatures.util.ActivityGroupSubgroupQuerySpecification;
import org.eclipse.incquery.uml.derivedfeatures.util.ActivityGroupSuperGroupQuerySpecification;
import org.eclipse.incquery.uml.derivedfeatures.util.ActivityNodeInGroupQuerySpecification;
import org.eclipse.incquery.uml.derivedfeatures.util.ActivityNodeQuerySpecification;
import org.eclipse.incquery.uml.derivedfeatures.util.AssociationEndTypeQuerySpecification;
import org.eclipse.incquery.uml.derivedfeatures.util.ClassExtensionQuerySpecification;
import org.eclipse.incquery.uml.derivedfeatures.util.ClassSuperClassQuerySpecification;
import org.eclipse.incquery.uml.derivedfeatures.util.ClassifierAllParentsQuerySpecification;
import org.eclipse.incquery.uml.derivedfeatures.util.ClassifierAttributeQuerySpecification;
import org.eclipse.incquery.uml.derivedfeatures.util.ClassifierFeatureQuerySpecification;
import org.eclipse.incquery.uml.derivedfeatures.util.ClassifierGeneralQuerySpecification;
import org.eclipse.incquery.uml.derivedfeatures.util.ClassifierParentsQuerySpecification;
import org.eclipse.incquery.uml.derivedfeatures.util.ConnectableElementEndQuerySpecification;
import org.eclipse.incquery.uml.derivedfeatures.util.ConnectorKindQuerySpecification;
import org.eclipse.incquery.uml.derivedfeatures.util.DeploymentTargetDeployedElementQuerySpecification;
import org.eclipse.incquery.uml.derivedfeatures.util.DirectedRelationshipSourceQuerySpecification;
import org.eclipse.incquery.uml.derivedfeatures.util.DirectedRelationshipTargetQuerySpecification;
import org.eclipse.incquery.uml.derivedfeatures.util.ElementOwnedElementQuerySpecification;
import org.eclipse.incquery.uml.derivedfeatures.util.ElementOwnerQuerySpecification;
import org.eclipse.incquery.uml.derivedfeatures.util.EncapsulatedClassifierOwnedPortQuerySpecification;
import org.eclipse.incquery.uml.derivedfeatures.util.ExtensionMetaclassEndQuerySpecification;
import org.eclipse.incquery.uml.derivedfeatures.util.ExtensionMetaclassQuerySpecification;
import org.eclipse.incquery.uml.derivedfeatures.util.FeatureFeaturingClassifierQuerySpecification;
import org.eclipse.incquery.uml.derivedfeatures.util.MessageMessageKindQuerySpecification;
import org.eclipse.incquery.uml.derivedfeatures.util.NamedElementClientDependencyQuerySpecification;
import org.eclipse.incquery.uml.derivedfeatures.util.NamedElementNamespaceQuerySpecification;
import org.eclipse.incquery.uml.derivedfeatures.util.NamedElementQualifiedNameQuerySpecification;
import org.eclipse.incquery.uml.derivedfeatures.util.NamespaceImportedMemberQuerySpecification;
import org.eclipse.incquery.uml.derivedfeatures.util.NamespaceMemberQuerySpecification;
import org.eclipse.incquery.uml.derivedfeatures.util.NamespaceOwnedMemberQuerySpecification;
import org.eclipse.incquery.uml.derivedfeatures.util.OpaqueExpressionResultQuerySpecification;
import org.eclipse.incquery.uml.derivedfeatures.util.OperationReturnResultQuerySpecification;
import org.eclipse.incquery.uml.derivedfeatures.util.PackageNestedPackageQuerySpecification;
import org.eclipse.incquery.uml.derivedfeatures.util.PackageNestingPackageQuerySpecification;
import org.eclipse.incquery.uml.derivedfeatures.util.PackageOwnedStereotypeQuerySpecification;
import org.eclipse.incquery.uml.derivedfeatures.util.PackageOwnedTypeQuerySpecification;
import org.eclipse.incquery.uml.derivedfeatures.util.PackageVisibleMemberQuerySpecification;
import org.eclipse.incquery.uml.derivedfeatures.util.PropertyIsCompositeQuerySpecification;
import org.eclipse.incquery.uml.derivedfeatures.util.ProtocolTransitionReferredQuerySpecification;
import org.eclipse.incquery.uml.derivedfeatures.util.RedefinableElementRedefinedElementQuerySpecification;
import org.eclipse.incquery.uml.derivedfeatures.util.RedefinableElementRedefinitionContextQuerySpecification;
import org.eclipse.incquery.uml.derivedfeatures.util.RedefinableTemplateSignatureInheritedParameterQuerySpecification;
import org.eclipse.incquery.uml.derivedfeatures.util.RelationshipRelatedElementQuerySpecification;
import org.eclipse.incquery.uml.derivedfeatures.util.StateIsCompositeQuerySpecification;
import org.eclipse.incquery.uml.derivedfeatures.util.StateIsComposite_0QuerySpecification;
import org.eclipse.incquery.uml.derivedfeatures.util.StateIsOrthogonalQuerySpecification;
import org.eclipse.incquery.uml.derivedfeatures.util.StructuredClassifierPartQuerySpecification;
import org.eclipse.incquery.uml.derivedfeatures.util.StructuredClassifierRoleQuerySpecification;
import org.eclipse.incquery.uml.derivedfeatures.util.TypePackageQuerySpecification;
import org.eclipse.incquery.uml.derivedfeatures.util.VertexIncomingQuerySpecification;
import org.eclipse.incquery.uml.derivedfeatures.util.VertexOutgoingQuerySpecification;

/**
 * A pattern group formed of all patterns defined in DerivedFeatures.eiq.
 * 
 * <p>Use the static instance as any {@link org.eclipse.incquery.runtime.api.IPatternGroup}, to conveniently prepare
 * an EMF-IncQuery engine for matching all patterns originally defined in file DerivedFeatures.eiq,
 * in order to achieve better performance than one-by-one on-demand matcher initialization.
 * 
 * <p> From package org.eclipse.incquery.uml.derivedfeatures, the group contains the definition of the following patterns: <ul>
 * <li>actionInput</li>
 * <li>actionOutput</li>
 * <li>activityGroup</li>
 * <li>activityNode</li>
 * <li>activityEdgeInGroup</li>
 * <li>activityGroupContainedEdge</li>
 * <li>activityGroupContainedNode</li>
 * <li>activityGroupSubgroup</li>
 * <li>activityGroupSuperGroup</li>
 * <li>activityNodeInGroup</li>
 * <li>classifierAttribute</li>
 * <li>classifierFeature</li>
 * <li>directedRelationshipSource</li>
 * <li>directedRelationshipTarget</li>
 * <li>elementOwnedElement</li>
 * <li>elementOwner</li>
 * <li>featureFeaturingClassifier</li>
 * <li>namedElementNamespace</li>
 * <li>namespaceMember</li>
 * <li>namespaceOwnedMember</li>
 * <li>redefinableElementRedefinedElement</li>
 * <li>redefinableElementRedefinitionContext</li>
 * <li>relationshipRelatedElement</li>
 * <li>structuredClassifierRole</li>
 * <li>associationEndType</li>
 * <li>connectableElementEnd</li>
 * <li>deploymentTargetDeployedElement</li>
 * <li>namedElementClientDependency</li>
 * <li>protocolTransitionReferred</li>
 * <li>vertexIncoming</li>
 * <li>vertexOutgoing</li>
 * <li>connectorKind</li>
 * <li>connectorKindDelegation</li>
 * <li>connectorEndPartWithPort</li>
 * <li>messageMessageKind</li>
 * <li>messageSendEvent</li>
 * <li>messageReceiveEvent</li>
 * <li>packageNestedPackage</li>
 * <li>packageOwnedStereotype</li>
 * <li>packageOwnedType</li>
 * <li>propertyIsComposite</li>
 * <li>stateIsComposite</li>
 * <li>stateIsComposite_0</li>
 * <li>typePackage</li>
 * <li>classExtension</li>
 * <li>typeOrAllParents</li>
 * <li>classSuperClass</li>
 * <li>classifierAllParents</li>
 * <li>classifierGeneral</li>
 * <li>encapsulatedClassifierOwnedPort</li>
 * <li>extensionMetaclass</li>
 * <li>extensionMetaclassEnd</li>
 * <li>extensionOwnedEnd</li>
 * <li>namedElementQualifiedName</li>
 * <li>namespaceImportedMember</li>
 * <li>opaqueExpressionResult</li>
 * <li>packageVisibleMember</li>
 * <li>packageDirectlyVisibleMember</li>
 * <li>packageAllImportedPackages</li>
 * <li>packageImportedPackage</li>
 * <li>packageNestingPackage</li>
 * <li>redefinableTemplateSignatureInheritedParameter</li>
 * <li>structuredClassifierPart</li>
 * <li>stateIsOrthogonal</li>
 * <li>classifierParents</li>
 * <li>operationReturnResult</li>
 * </ul>
 * 
 * @see IPatternGroup
 * 
 */
@SuppressWarnings("all")
public final class DerivedFeatures extends BaseGeneratedPatternGroup {
  /**
   * Access the pattern group.
   * 
   * @return the singleton instance of the group
   * @throws IncQueryException if there was an error loading the generated code of pattern specifications
   * 
   */
  public static DerivedFeatures instance() throws IncQueryException {
    if (INSTANCE == null) {
    	INSTANCE = new DerivedFeatures();
    }
    return INSTANCE;
  }
  
  private static DerivedFeatures INSTANCE;
  
  private DerivedFeatures() throws IncQueryException {
    querySpecifications.add(ActionInputQuerySpecification.instance());
    querySpecifications.add(ActionOutputQuerySpecification.instance());
    querySpecifications.add(ActivityGroupQuerySpecification.instance());
    querySpecifications.add(ActivityNodeQuerySpecification.instance());
    querySpecifications.add(ActivityEdgeInGroupQuerySpecification.instance());
    querySpecifications.add(ActivityGroupContainedEdgeQuerySpecification.instance());
    querySpecifications.add(ActivityGroupContainedNodeQuerySpecification.instance());
    querySpecifications.add(ActivityGroupSubgroupQuerySpecification.instance());
    querySpecifications.add(ActivityGroupSuperGroupQuerySpecification.instance());
    querySpecifications.add(ActivityNodeInGroupQuerySpecification.instance());
    querySpecifications.add(ClassifierAttributeQuerySpecification.instance());
    querySpecifications.add(ClassifierFeatureQuerySpecification.instance());
    querySpecifications.add(DirectedRelationshipSourceQuerySpecification.instance());
    querySpecifications.add(DirectedRelationshipTargetQuerySpecification.instance());
    querySpecifications.add(ElementOwnedElementQuerySpecification.instance());
    querySpecifications.add(ElementOwnerQuerySpecification.instance());
    querySpecifications.add(FeatureFeaturingClassifierQuerySpecification.instance());
    querySpecifications.add(NamedElementNamespaceQuerySpecification.instance());
    querySpecifications.add(NamespaceMemberQuerySpecification.instance());
    querySpecifications.add(NamespaceOwnedMemberQuerySpecification.instance());
    querySpecifications.add(RedefinableElementRedefinedElementQuerySpecification.instance());
    querySpecifications.add(RedefinableElementRedefinitionContextQuerySpecification.instance());
    querySpecifications.add(RelationshipRelatedElementQuerySpecification.instance());
    querySpecifications.add(StructuredClassifierRoleQuerySpecification.instance());
    querySpecifications.add(AssociationEndTypeQuerySpecification.instance());
    querySpecifications.add(ConnectableElementEndQuerySpecification.instance());
    querySpecifications.add(DeploymentTargetDeployedElementQuerySpecification.instance());
    querySpecifications.add(NamedElementClientDependencyQuerySpecification.instance());
    querySpecifications.add(ProtocolTransitionReferredQuerySpecification.instance());
    querySpecifications.add(VertexIncomingQuerySpecification.instance());
    querySpecifications.add(VertexOutgoingQuerySpecification.instance());
    querySpecifications.add(ConnectorKindQuerySpecification.instance());
    querySpecifications.add(MessageMessageKindQuerySpecification.instance());
    querySpecifications.add(PackageNestedPackageQuerySpecification.instance());
    querySpecifications.add(PackageOwnedStereotypeQuerySpecification.instance());
    querySpecifications.add(PackageOwnedTypeQuerySpecification.instance());
    querySpecifications.add(PropertyIsCompositeQuerySpecification.instance());
    querySpecifications.add(StateIsCompositeQuerySpecification.instance());
    querySpecifications.add(StateIsComposite_0QuerySpecification.instance());
    querySpecifications.add(TypePackageQuerySpecification.instance());
    querySpecifications.add(ClassExtensionQuerySpecification.instance());
    querySpecifications.add(ClassSuperClassQuerySpecification.instance());
    querySpecifications.add(ClassifierAllParentsQuerySpecification.instance());
    querySpecifications.add(ClassifierGeneralQuerySpecification.instance());
    querySpecifications.add(EncapsulatedClassifierOwnedPortQuerySpecification.instance());
    querySpecifications.add(ExtensionMetaclassQuerySpecification.instance());
    querySpecifications.add(ExtensionMetaclassEndQuerySpecification.instance());
    querySpecifications.add(NamedElementQualifiedNameQuerySpecification.instance());
    querySpecifications.add(NamespaceImportedMemberQuerySpecification.instance());
    querySpecifications.add(OpaqueExpressionResultQuerySpecification.instance());
    querySpecifications.add(PackageVisibleMemberQuerySpecification.instance());
    querySpecifications.add(PackageNestingPackageQuerySpecification.instance());
    querySpecifications.add(RedefinableTemplateSignatureInheritedParameterQuerySpecification.instance());
    querySpecifications.add(StructuredClassifierPartQuerySpecification.instance());
    querySpecifications.add(StateIsOrthogonalQuerySpecification.instance());
    querySpecifications.add(ClassifierParentsQuerySpecification.instance());
    querySpecifications.add(OperationReturnResultQuerySpecification.instance());
  }
  
  public ActionInputQuerySpecification getActionInput() throws IncQueryException {
    return ActionInputQuerySpecification.instance();
  }
  
  public ActionInputMatcher getActionInput(final IncQueryEngine engine) throws IncQueryException {
    return ActionInputMatcher.on(engine);
  }
  
  public ActionOutputQuerySpecification getActionOutput() throws IncQueryException {
    return ActionOutputQuerySpecification.instance();
  }
  
  public ActionOutputMatcher getActionOutput(final IncQueryEngine engine) throws IncQueryException {
    return ActionOutputMatcher.on(engine);
  }
  
  public ActivityGroupQuerySpecification getActivityGroup() throws IncQueryException {
    return ActivityGroupQuerySpecification.instance();
  }
  
  public ActivityGroupMatcher getActivityGroup(final IncQueryEngine engine) throws IncQueryException {
    return ActivityGroupMatcher.on(engine);
  }
  
  public ActivityNodeQuerySpecification getActivityNode() throws IncQueryException {
    return ActivityNodeQuerySpecification.instance();
  }
  
  public ActivityNodeMatcher getActivityNode(final IncQueryEngine engine) throws IncQueryException {
    return ActivityNodeMatcher.on(engine);
  }
  
  public ActivityEdgeInGroupQuerySpecification getActivityEdgeInGroup() throws IncQueryException {
    return ActivityEdgeInGroupQuerySpecification.instance();
  }
  
  public ActivityEdgeInGroupMatcher getActivityEdgeInGroup(final IncQueryEngine engine) throws IncQueryException {
    return ActivityEdgeInGroupMatcher.on(engine);
  }
  
  public ActivityGroupContainedEdgeQuerySpecification getActivityGroupContainedEdge() throws IncQueryException {
    return ActivityGroupContainedEdgeQuerySpecification.instance();
  }
  
  public ActivityGroupContainedEdgeMatcher getActivityGroupContainedEdge(final IncQueryEngine engine) throws IncQueryException {
    return ActivityGroupContainedEdgeMatcher.on(engine);
  }
  
  public ActivityGroupContainedNodeQuerySpecification getActivityGroupContainedNode() throws IncQueryException {
    return ActivityGroupContainedNodeQuerySpecification.instance();
  }
  
  public ActivityGroupContainedNodeMatcher getActivityGroupContainedNode(final IncQueryEngine engine) throws IncQueryException {
    return ActivityGroupContainedNodeMatcher.on(engine);
  }
  
  public ActivityGroupSubgroupQuerySpecification getActivityGroupSubgroup() throws IncQueryException {
    return ActivityGroupSubgroupQuerySpecification.instance();
  }
  
  public ActivityGroupSubgroupMatcher getActivityGroupSubgroup(final IncQueryEngine engine) throws IncQueryException {
    return ActivityGroupSubgroupMatcher.on(engine);
  }
  
  public ActivityGroupSuperGroupQuerySpecification getActivityGroupSuperGroup() throws IncQueryException {
    return ActivityGroupSuperGroupQuerySpecification.instance();
  }
  
  public ActivityGroupSuperGroupMatcher getActivityGroupSuperGroup(final IncQueryEngine engine) throws IncQueryException {
    return ActivityGroupSuperGroupMatcher.on(engine);
  }
  
  public ActivityNodeInGroupQuerySpecification getActivityNodeInGroup() throws IncQueryException {
    return ActivityNodeInGroupQuerySpecification.instance();
  }
  
  public ActivityNodeInGroupMatcher getActivityNodeInGroup(final IncQueryEngine engine) throws IncQueryException {
    return ActivityNodeInGroupMatcher.on(engine);
  }
  
  public ClassifierAttributeQuerySpecification getClassifierAttribute() throws IncQueryException {
    return ClassifierAttributeQuerySpecification.instance();
  }
  
  public ClassifierAttributeMatcher getClassifierAttribute(final IncQueryEngine engine) throws IncQueryException {
    return ClassifierAttributeMatcher.on(engine);
  }
  
  public ClassifierFeatureQuerySpecification getClassifierFeature() throws IncQueryException {
    return ClassifierFeatureQuerySpecification.instance();
  }
  
  public ClassifierFeatureMatcher getClassifierFeature(final IncQueryEngine engine) throws IncQueryException {
    return ClassifierFeatureMatcher.on(engine);
  }
  
  public DirectedRelationshipSourceQuerySpecification getDirectedRelationshipSource() throws IncQueryException {
    return DirectedRelationshipSourceQuerySpecification.instance();
  }
  
  public DirectedRelationshipSourceMatcher getDirectedRelationshipSource(final IncQueryEngine engine) throws IncQueryException {
    return DirectedRelationshipSourceMatcher.on(engine);
  }
  
  public DirectedRelationshipTargetQuerySpecification getDirectedRelationshipTarget() throws IncQueryException {
    return DirectedRelationshipTargetQuerySpecification.instance();
  }
  
  public DirectedRelationshipTargetMatcher getDirectedRelationshipTarget(final IncQueryEngine engine) throws IncQueryException {
    return DirectedRelationshipTargetMatcher.on(engine);
  }
  
  public ElementOwnedElementQuerySpecification getElementOwnedElement() throws IncQueryException {
    return ElementOwnedElementQuerySpecification.instance();
  }
  
  public ElementOwnedElementMatcher getElementOwnedElement(final IncQueryEngine engine) throws IncQueryException {
    return ElementOwnedElementMatcher.on(engine);
  }
  
  public ElementOwnerQuerySpecification getElementOwner() throws IncQueryException {
    return ElementOwnerQuerySpecification.instance();
  }
  
  public ElementOwnerMatcher getElementOwner(final IncQueryEngine engine) throws IncQueryException {
    return ElementOwnerMatcher.on(engine);
  }
  
  public FeatureFeaturingClassifierQuerySpecification getFeatureFeaturingClassifier() throws IncQueryException {
    return FeatureFeaturingClassifierQuerySpecification.instance();
  }
  
  public FeatureFeaturingClassifierMatcher getFeatureFeaturingClassifier(final IncQueryEngine engine) throws IncQueryException {
    return FeatureFeaturingClassifierMatcher.on(engine);
  }
  
  public NamedElementNamespaceQuerySpecification getNamedElementNamespace() throws IncQueryException {
    return NamedElementNamespaceQuerySpecification.instance();
  }
  
  public NamedElementNamespaceMatcher getNamedElementNamespace(final IncQueryEngine engine) throws IncQueryException {
    return NamedElementNamespaceMatcher.on(engine);
  }
  
  public NamespaceMemberQuerySpecification getNamespaceMember() throws IncQueryException {
    return NamespaceMemberQuerySpecification.instance();
  }
  
  public NamespaceMemberMatcher getNamespaceMember(final IncQueryEngine engine) throws IncQueryException {
    return NamespaceMemberMatcher.on(engine);
  }
  
  public NamespaceOwnedMemberQuerySpecification getNamespaceOwnedMember() throws IncQueryException {
    return NamespaceOwnedMemberQuerySpecification.instance();
  }
  
  public NamespaceOwnedMemberMatcher getNamespaceOwnedMember(final IncQueryEngine engine) throws IncQueryException {
    return NamespaceOwnedMemberMatcher.on(engine);
  }
  
  public RedefinableElementRedefinedElementQuerySpecification getRedefinableElementRedefinedElement() throws IncQueryException {
    return RedefinableElementRedefinedElementQuerySpecification.instance();
  }
  
  public RedefinableElementRedefinedElementMatcher getRedefinableElementRedefinedElement(final IncQueryEngine engine) throws IncQueryException {
    return RedefinableElementRedefinedElementMatcher.on(engine);
  }
  
  public RedefinableElementRedefinitionContextQuerySpecification getRedefinableElementRedefinitionContext() throws IncQueryException {
    return RedefinableElementRedefinitionContextQuerySpecification.instance();
  }
  
  public RedefinableElementRedefinitionContextMatcher getRedefinableElementRedefinitionContext(final IncQueryEngine engine) throws IncQueryException {
    return RedefinableElementRedefinitionContextMatcher.on(engine);
  }
  
  public RelationshipRelatedElementQuerySpecification getRelationshipRelatedElement() throws IncQueryException {
    return RelationshipRelatedElementQuerySpecification.instance();
  }
  
  public RelationshipRelatedElementMatcher getRelationshipRelatedElement(final IncQueryEngine engine) throws IncQueryException {
    return RelationshipRelatedElementMatcher.on(engine);
  }
  
  public StructuredClassifierRoleQuerySpecification getStructuredClassifierRole() throws IncQueryException {
    return StructuredClassifierRoleQuerySpecification.instance();
  }
  
  public StructuredClassifierRoleMatcher getStructuredClassifierRole(final IncQueryEngine engine) throws IncQueryException {
    return StructuredClassifierRoleMatcher.on(engine);
  }
  
  public AssociationEndTypeQuerySpecification getAssociationEndType() throws IncQueryException {
    return AssociationEndTypeQuerySpecification.instance();
  }
  
  public AssociationEndTypeMatcher getAssociationEndType(final IncQueryEngine engine) throws IncQueryException {
    return AssociationEndTypeMatcher.on(engine);
  }
  
  public ConnectableElementEndQuerySpecification getConnectableElementEnd() throws IncQueryException {
    return ConnectableElementEndQuerySpecification.instance();
  }
  
  public ConnectableElementEndMatcher getConnectableElementEnd(final IncQueryEngine engine) throws IncQueryException {
    return ConnectableElementEndMatcher.on(engine);
  }
  
  public DeploymentTargetDeployedElementQuerySpecification getDeploymentTargetDeployedElement() throws IncQueryException {
    return DeploymentTargetDeployedElementQuerySpecification.instance();
  }
  
  public DeploymentTargetDeployedElementMatcher getDeploymentTargetDeployedElement(final IncQueryEngine engine) throws IncQueryException {
    return DeploymentTargetDeployedElementMatcher.on(engine);
  }
  
  public NamedElementClientDependencyQuerySpecification getNamedElementClientDependency() throws IncQueryException {
    return NamedElementClientDependencyQuerySpecification.instance();
  }
  
  public NamedElementClientDependencyMatcher getNamedElementClientDependency(final IncQueryEngine engine) throws IncQueryException {
    return NamedElementClientDependencyMatcher.on(engine);
  }
  
  public ProtocolTransitionReferredQuerySpecification getProtocolTransitionReferred() throws IncQueryException {
    return ProtocolTransitionReferredQuerySpecification.instance();
  }
  
  public ProtocolTransitionReferredMatcher getProtocolTransitionReferred(final IncQueryEngine engine) throws IncQueryException {
    return ProtocolTransitionReferredMatcher.on(engine);
  }
  
  public VertexIncomingQuerySpecification getVertexIncoming() throws IncQueryException {
    return VertexIncomingQuerySpecification.instance();
  }
  
  public VertexIncomingMatcher getVertexIncoming(final IncQueryEngine engine) throws IncQueryException {
    return VertexIncomingMatcher.on(engine);
  }
  
  public VertexOutgoingQuerySpecification getVertexOutgoing() throws IncQueryException {
    return VertexOutgoingQuerySpecification.instance();
  }
  
  public VertexOutgoingMatcher getVertexOutgoing(final IncQueryEngine engine) throws IncQueryException {
    return VertexOutgoingMatcher.on(engine);
  }
  
  public ConnectorKindQuerySpecification getConnectorKind() throws IncQueryException {
    return ConnectorKindQuerySpecification.instance();
  }
  
  public ConnectorKindMatcher getConnectorKind(final IncQueryEngine engine) throws IncQueryException {
    return ConnectorKindMatcher.on(engine);
  }
  
  public MessageMessageKindQuerySpecification getMessageMessageKind() throws IncQueryException {
    return MessageMessageKindQuerySpecification.instance();
  }
  
  public MessageMessageKindMatcher getMessageMessageKind(final IncQueryEngine engine) throws IncQueryException {
    return MessageMessageKindMatcher.on(engine);
  }
  
  public PackageNestedPackageQuerySpecification getPackageNestedPackage() throws IncQueryException {
    return PackageNestedPackageQuerySpecification.instance();
  }
  
  public PackageNestedPackageMatcher getPackageNestedPackage(final IncQueryEngine engine) throws IncQueryException {
    return PackageNestedPackageMatcher.on(engine);
  }
  
  public PackageOwnedStereotypeQuerySpecification getPackageOwnedStereotype() throws IncQueryException {
    return PackageOwnedStereotypeQuerySpecification.instance();
  }
  
  public PackageOwnedStereotypeMatcher getPackageOwnedStereotype(final IncQueryEngine engine) throws IncQueryException {
    return PackageOwnedStereotypeMatcher.on(engine);
  }
  
  public PackageOwnedTypeQuerySpecification getPackageOwnedType() throws IncQueryException {
    return PackageOwnedTypeQuerySpecification.instance();
  }
  
  public PackageOwnedTypeMatcher getPackageOwnedType(final IncQueryEngine engine) throws IncQueryException {
    return PackageOwnedTypeMatcher.on(engine);
  }
  
  public PropertyIsCompositeQuerySpecification getPropertyIsComposite() throws IncQueryException {
    return PropertyIsCompositeQuerySpecification.instance();
  }
  
  public PropertyIsCompositeMatcher getPropertyIsComposite(final IncQueryEngine engine) throws IncQueryException {
    return PropertyIsCompositeMatcher.on(engine);
  }
  
  public StateIsCompositeQuerySpecification getStateIsComposite() throws IncQueryException {
    return StateIsCompositeQuerySpecification.instance();
  }
  
  public StateIsCompositeMatcher getStateIsComposite(final IncQueryEngine engine) throws IncQueryException {
    return StateIsCompositeMatcher.on(engine);
  }
  
  public StateIsComposite_0QuerySpecification getStateIsComposite_0() throws IncQueryException {
    return StateIsComposite_0QuerySpecification.instance();
  }
  
  public StateIsComposite_0Matcher getStateIsComposite_0(final IncQueryEngine engine) throws IncQueryException {
    return StateIsComposite_0Matcher.on(engine);
  }
  
  public TypePackageQuerySpecification getTypePackage() throws IncQueryException {
    return TypePackageQuerySpecification.instance();
  }
  
  public TypePackageMatcher getTypePackage(final IncQueryEngine engine) throws IncQueryException {
    return TypePackageMatcher.on(engine);
  }
  
  public ClassExtensionQuerySpecification getClassExtension() throws IncQueryException {
    return ClassExtensionQuerySpecification.instance();
  }
  
  public ClassExtensionMatcher getClassExtension(final IncQueryEngine engine) throws IncQueryException {
    return ClassExtensionMatcher.on(engine);
  }
  
  public ClassSuperClassQuerySpecification getClassSuperClass() throws IncQueryException {
    return ClassSuperClassQuerySpecification.instance();
  }
  
  public ClassSuperClassMatcher getClassSuperClass(final IncQueryEngine engine) throws IncQueryException {
    return ClassSuperClassMatcher.on(engine);
  }
  
  public ClassifierAllParentsQuerySpecification getClassifierAllParents() throws IncQueryException {
    return ClassifierAllParentsQuerySpecification.instance();
  }
  
  public ClassifierAllParentsMatcher getClassifierAllParents(final IncQueryEngine engine) throws IncQueryException {
    return ClassifierAllParentsMatcher.on(engine);
  }
  
  public ClassifierGeneralQuerySpecification getClassifierGeneral() throws IncQueryException {
    return ClassifierGeneralQuerySpecification.instance();
  }
  
  public ClassifierGeneralMatcher getClassifierGeneral(final IncQueryEngine engine) throws IncQueryException {
    return ClassifierGeneralMatcher.on(engine);
  }
  
  public EncapsulatedClassifierOwnedPortQuerySpecification getEncapsulatedClassifierOwnedPort() throws IncQueryException {
    return EncapsulatedClassifierOwnedPortQuerySpecification.instance();
  }
  
  public EncapsulatedClassifierOwnedPortMatcher getEncapsulatedClassifierOwnedPort(final IncQueryEngine engine) throws IncQueryException {
    return EncapsulatedClassifierOwnedPortMatcher.on(engine);
  }
  
  public ExtensionMetaclassQuerySpecification getExtensionMetaclass() throws IncQueryException {
    return ExtensionMetaclassQuerySpecification.instance();
  }
  
  public ExtensionMetaclassMatcher getExtensionMetaclass(final IncQueryEngine engine) throws IncQueryException {
    return ExtensionMetaclassMatcher.on(engine);
  }
  
  public ExtensionMetaclassEndQuerySpecification getExtensionMetaclassEnd() throws IncQueryException {
    return ExtensionMetaclassEndQuerySpecification.instance();
  }
  
  public ExtensionMetaclassEndMatcher getExtensionMetaclassEnd(final IncQueryEngine engine) throws IncQueryException {
    return ExtensionMetaclassEndMatcher.on(engine);
  }
  
  public NamedElementQualifiedNameQuerySpecification getNamedElementQualifiedName() throws IncQueryException {
    return NamedElementQualifiedNameQuerySpecification.instance();
  }
  
  public NamedElementQualifiedNameMatcher getNamedElementQualifiedName(final IncQueryEngine engine) throws IncQueryException {
    return NamedElementQualifiedNameMatcher.on(engine);
  }
  
  public NamespaceImportedMemberQuerySpecification getNamespaceImportedMember() throws IncQueryException {
    return NamespaceImportedMemberQuerySpecification.instance();
  }
  
  public NamespaceImportedMemberMatcher getNamespaceImportedMember(final IncQueryEngine engine) throws IncQueryException {
    return NamespaceImportedMemberMatcher.on(engine);
  }
  
  public OpaqueExpressionResultQuerySpecification getOpaqueExpressionResult() throws IncQueryException {
    return OpaqueExpressionResultQuerySpecification.instance();
  }
  
  public OpaqueExpressionResultMatcher getOpaqueExpressionResult(final IncQueryEngine engine) throws IncQueryException {
    return OpaqueExpressionResultMatcher.on(engine);
  }
  
  public PackageVisibleMemberQuerySpecification getPackageVisibleMember() throws IncQueryException {
    return PackageVisibleMemberQuerySpecification.instance();
  }
  
  public PackageVisibleMemberMatcher getPackageVisibleMember(final IncQueryEngine engine) throws IncQueryException {
    return PackageVisibleMemberMatcher.on(engine);
  }
  
  public PackageNestingPackageQuerySpecification getPackageNestingPackage() throws IncQueryException {
    return PackageNestingPackageQuerySpecification.instance();
  }
  
  public PackageNestingPackageMatcher getPackageNestingPackage(final IncQueryEngine engine) throws IncQueryException {
    return PackageNestingPackageMatcher.on(engine);
  }
  
  public RedefinableTemplateSignatureInheritedParameterQuerySpecification getRedefinableTemplateSignatureInheritedParameter() throws IncQueryException {
    return RedefinableTemplateSignatureInheritedParameterQuerySpecification.instance();
  }
  
  public RedefinableTemplateSignatureInheritedParameterMatcher getRedefinableTemplateSignatureInheritedParameter(final IncQueryEngine engine) throws IncQueryException {
    return RedefinableTemplateSignatureInheritedParameterMatcher.on(engine);
  }
  
  public StructuredClassifierPartQuerySpecification getStructuredClassifierPart() throws IncQueryException {
    return StructuredClassifierPartQuerySpecification.instance();
  }
  
  public StructuredClassifierPartMatcher getStructuredClassifierPart(final IncQueryEngine engine) throws IncQueryException {
    return StructuredClassifierPartMatcher.on(engine);
  }
  
  public StateIsOrthogonalQuerySpecification getStateIsOrthogonal() throws IncQueryException {
    return StateIsOrthogonalQuerySpecification.instance();
  }
  
  public StateIsOrthogonalMatcher getStateIsOrthogonal(final IncQueryEngine engine) throws IncQueryException {
    return StateIsOrthogonalMatcher.on(engine);
  }
  
  public ClassifierParentsQuerySpecification getClassifierParents() throws IncQueryException {
    return ClassifierParentsQuerySpecification.instance();
  }
  
  public ClassifierParentsMatcher getClassifierParents(final IncQueryEngine engine) throws IncQueryException {
    return ClassifierParentsMatcher.on(engine);
  }
  
  public OperationReturnResultQuerySpecification getOperationReturnResult() throws IncQueryException {
    return OperationReturnResultQuerySpecification.instance();
  }
  
  public OperationReturnResultMatcher getOperationReturnResult(final IncQueryEngine engine) throws IncQueryException {
    return OperationReturnResultMatcher.on(engine);
  }
}
