package org.eclipse.incquery.uml.derivedfeatures;

import java.util.Collection;
import java.util.HashSet;
import java.util.Set;
import org.apache.log4j.Logger;
import org.eclipse.emf.common.notify.Notifier;
import org.eclipse.incquery.runtime.api.IMatchProcessor;
import org.eclipse.incquery.runtime.api.IQuerySpecification;
import org.eclipse.incquery.runtime.api.IncQueryEngine;
import org.eclipse.incquery.runtime.api.impl.BaseMatcher;
import org.eclipse.incquery.runtime.exception.IncQueryException;
import org.eclipse.incquery.runtime.matchers.tuple.Tuple;
import org.eclipse.incquery.runtime.util.IncQueryLoggingUtil;
import org.eclipse.incquery.uml.derivedfeatures.NamespaceOwnedMemberMatch;
import org.eclipse.incquery.uml.derivedfeatures.util.NamespaceOwnedMemberQuerySpecification;
import org.eclipse.uml2.uml.NamedElement;
import org.eclipse.uml2.uml.Namespace;

/**
 * Generated pattern matcher API of the org.eclipse.incquery.uml.derivedfeatures.namespaceOwnedMember pattern,
 * providing pattern-specific query methods.
 * 
 * <p>Use the pattern matcher on a given model via {@link #on(IncQueryEngine)},
 * e.g. in conjunction with {@link IncQueryEngine#on(Notifier)}.
 * 
 * <p>Matches of the pattern will be represented as {@link NamespaceOwnedMemberMatch}.
 * 
 * <p>Original source:
 * <code><pre>
 * {@literal @}Surrogate(feature = "ownedMember")
 * {@literal @}QueryExplorer(checked = false)
 * pattern namespaceOwnedMember(source: Namespace, target: NamedElement)
 * { Activity.variable(source, target); } or 
 * { Artifact.nestedArtifact(source, target); } or 
 * { Artifact.ownedAttribute(source, target); } or 
 * { Artifact.ownedOperation(source, target); } or 
 * { Association.ownedEnd(source, target); } or 
 * { Behavior.ownedParameter(source, target); } or 
 * { Behavior.ownedParameterSet(source, target); } or 
 * { BehavioralFeature.ownedParameter(source, target); } or 
 * { BehavioralFeature.ownedParameterSet(source, target); } or 
 * { BehavioredClassifier.ownedBehavior(source, target); } or 
 * { Class.nestedClassifier(source, target); } or 
 * { Class.ownedOperation(source, target); } or 
 * { Class.ownedReception(source, target); } or 
 * { Classifier.ownedUseCase(source, target); } or 
 * { Component.packagedElement(source, target); } or 
 * { DataType.ownedAttribute(source, target); } or 
 * { DataType.ownedOperation(source, target); } or 
 * { Enumeration.ownedLiteral(source, target); } or 
 * { Interaction.formalGate(source, target); } or 
 * { Interaction.fragment(source, target); } or 
 * { Interaction.lifeline(source, target); } or 
 * { Interaction.message(source, target); } or 
 * { InteractionOperand.fragment(source, target); } or 
 * { Interface.nestedClassifier(source, target); } or 
 * { Interface.ownedAttribute(source, target); } or 
 * { Interface.ownedOperation(source, target); } or 
 * { Interface.ownedReception(source, target); } or 
 * { Interface.protocol(source, target); } or 
 * { Namespace.ownedRule(source, target); } or 
 * { Node.nestedNode(source, target); } or 
 * { Package.packagedElement(source, target); } or 
 * { Region.subvertex(source, target); } or 
 * { Region.transition(source, target); } or 
 * { Signal.ownedAttribute(source, target); } or 
 * { State.connection(source, target); } or 
 * { State.connectionPoint(source, target); } or 
 * { State.region(source, target); } or 
 * { StateMachine.connectionPoint(source, target); } or 
 * { StateMachine.region(source, target); } or 
 * { StructuredActivityNode.variable(source, target); } or 
 * { StructuredClassifier.ownedAttribute(source, target); } or 
 * { StructuredClassifier.ownedConnector(source, target); } or 
 * { UseCase.extend(source, target); } or 
 * { UseCase.extensionPoint(source, target); } or 
 * { UseCase.include(source, target); }
 * </pre></code>
 * 
 * @see NamespaceOwnedMemberMatch
 * @see NamespaceOwnedMemberProcessor
 * @see NamespaceOwnedMemberQuerySpecification
 * 
 */
@SuppressWarnings("all")
public class NamespaceOwnedMemberMatcher extends BaseMatcher<NamespaceOwnedMemberMatch> {
  /**
   * Initializes the pattern matcher within an existing EMF-IncQuery engine.
   * If the pattern matcher is already constructed in the engine, only a light-weight reference is returned.
   * The match set will be incrementally refreshed upon updates.
   * @param engine the existing EMF-IncQuery engine in which this matcher will be created.
   * @throws IncQueryException if an error occurs during pattern matcher creation
   * 
   */
  public static NamespaceOwnedMemberMatcher on(final IncQueryEngine engine) throws IncQueryException {
    // check if matcher already exists
    NamespaceOwnedMemberMatcher matcher = engine.getExistingMatcher(querySpecification());
    if (matcher == null) {
    	matcher = new NamespaceOwnedMemberMatcher(engine);
    	// do not have to "put" it into engine.matchers, reportMatcherInitialized() will take care of it
    }
    return matcher;
  }
  
  private final static int POSITION_SOURCE = 0;
  
  private final static int POSITION_TARGET = 1;
  
  private final static Logger LOGGER = IncQueryLoggingUtil.getLogger(NamespaceOwnedMemberMatcher.class);
  
  /**
   * Initializes the pattern matcher over a given EMF model root (recommended: Resource or ResourceSet).
   * If a pattern matcher is already constructed with the same root, only a light-weight reference is returned.
   * The scope of pattern matching will be the given EMF model root and below (see FAQ for more precise definition).
   * The match set will be incrementally refreshed upon updates from this scope.
   * <p>The matcher will be created within the managed {@link IncQueryEngine} belonging to the EMF model root, so
   * multiple matchers will reuse the same engine and benefit from increased performance and reduced memory footprint.
   * @param emfRoot the root of the EMF containment hierarchy where the pattern matcher will operate. Recommended: Resource or ResourceSet.
   * @throws IncQueryException if an error occurs during pattern matcher creation
   * @deprecated use {@link #on(IncQueryEngine)} instead, e.g. in conjunction with {@link IncQueryEngine#on(Notifier)}
   * 
   */
  @Deprecated
  public NamespaceOwnedMemberMatcher(final Notifier emfRoot) throws IncQueryException {
    this(IncQueryEngine.on(emfRoot));
  }
  
  /**
   * Initializes the pattern matcher within an existing EMF-IncQuery engine.
   * If the pattern matcher is already constructed in the engine, only a light-weight reference is returned.
   * The match set will be incrementally refreshed upon updates.
   * @param engine the existing EMF-IncQuery engine in which this matcher will be created.
   * @throws IncQueryException if an error occurs during pattern matcher creation
   * @deprecated use {@link #on(IncQueryEngine)} instead
   * 
   */
  @Deprecated
  public NamespaceOwnedMemberMatcher(final IncQueryEngine engine) throws IncQueryException {
    super(engine, querySpecification());
  }
  
  /**
   * Returns the set of all matches of the pattern that conform to the given fixed values of some parameters.
   * @param pSource the fixed value of pattern parameter source, or null if not bound.
   * @param pTarget the fixed value of pattern parameter target, or null if not bound.
   * @return matches represented as a NamespaceOwnedMemberMatch object.
   * 
   */
  public Collection<NamespaceOwnedMemberMatch> getAllMatches(final Namespace pSource, final NamedElement pTarget) {
    return rawGetAllMatches(new Object[]{pSource, pTarget});
  }
  
  /**
   * Returns an arbitrarily chosen match of the pattern that conforms to the given fixed values of some parameters.
   * Neither determinism nor randomness of selection is guaranteed.
   * @param pSource the fixed value of pattern parameter source, or null if not bound.
   * @param pTarget the fixed value of pattern parameter target, or null if not bound.
   * @return a match represented as a NamespaceOwnedMemberMatch object, or null if no match is found.
   * 
   */
  public NamespaceOwnedMemberMatch getOneArbitraryMatch(final Namespace pSource, final NamedElement pTarget) {
    return rawGetOneArbitraryMatch(new Object[]{pSource, pTarget});
  }
  
  /**
   * Indicates whether the given combination of specified pattern parameters constitute a valid pattern match,
   * under any possible substitution of the unspecified parameters (if any).
   * @param pSource the fixed value of pattern parameter source, or null if not bound.
   * @param pTarget the fixed value of pattern parameter target, or null if not bound.
   * @return true if the input is a valid (partial) match of the pattern.
   * 
   */
  public boolean hasMatch(final Namespace pSource, final NamedElement pTarget) {
    return rawHasMatch(new Object[]{pSource, pTarget});
  }
  
  /**
   * Returns the number of all matches of the pattern that conform to the given fixed values of some parameters.
   * @param pSource the fixed value of pattern parameter source, or null if not bound.
   * @param pTarget the fixed value of pattern parameter target, or null if not bound.
   * @return the number of pattern matches found.
   * 
   */
  public int countMatches(final Namespace pSource, final NamedElement pTarget) {
    return rawCountMatches(new Object[]{pSource, pTarget});
  }
  
  /**
   * Executes the given processor on each match of the pattern that conforms to the given fixed values of some parameters.
   * @param pSource the fixed value of pattern parameter source, or null if not bound.
   * @param pTarget the fixed value of pattern parameter target, or null if not bound.
   * @param processor the action that will process each pattern match.
   * 
   */
  public void forEachMatch(final Namespace pSource, final NamedElement pTarget, final IMatchProcessor<? super NamespaceOwnedMemberMatch> processor) {
    rawForEachMatch(new Object[]{pSource, pTarget}, processor);
  }
  
  /**
   * Executes the given processor on an arbitrarily chosen match of the pattern that conforms to the given fixed values of some parameters.
   * Neither determinism nor randomness of selection is guaranteed.
   * @param pSource the fixed value of pattern parameter source, or null if not bound.
   * @param pTarget the fixed value of pattern parameter target, or null if not bound.
   * @param processor the action that will process the selected match.
   * @return true if the pattern has at least one match with the given parameter values, false if the processor was not invoked
   * 
   */
  public boolean forOneArbitraryMatch(final Namespace pSource, final NamedElement pTarget, final IMatchProcessor<? super NamespaceOwnedMemberMatch> processor) {
    return rawForOneArbitraryMatch(new Object[]{pSource, pTarget}, processor);
  }
  
  /**
   * Returns a new (partial) match.
   * This can be used e.g. to call the matcher with a partial match.
   * <p>The returned match will be immutable. Use {@link #newEmptyMatch()} to obtain a mutable match object.
   * @param pSource the fixed value of pattern parameter source, or null if not bound.
   * @param pTarget the fixed value of pattern parameter target, or null if not bound.
   * @return the (partial) match object.
   * 
   */
  public NamespaceOwnedMemberMatch newMatch(final Namespace pSource, final NamedElement pTarget) {
    return NamespaceOwnedMemberMatch.newMatch(pSource, pTarget);
  }
  
  /**
   * Retrieve the set of values that occur in matches for source.
   * @return the Set of all values, null if no parameter with the given name exists, empty set if there are no matches
   * 
   */
  protected Set<Namespace> rawAccumulateAllValuesOfsource(final Object[] parameters) {
    Set<Namespace> results = new HashSet<Namespace>();
    rawAccumulateAllValues(POSITION_SOURCE, parameters, results);
    return results;
  }
  
  /**
   * Retrieve the set of values that occur in matches for source.
   * @return the Set of all values, null if no parameter with the given name exists, empty set if there are no matches
   * 
   */
  public Set<Namespace> getAllValuesOfsource() {
    return rawAccumulateAllValuesOfsource(emptyArray());
  }
  
  /**
   * Retrieve the set of values that occur in matches for source.
   * @return the Set of all values, null if no parameter with the given name exists, empty set if there are no matches
   * 
   */
  public Set<Namespace> getAllValuesOfsource(final NamespaceOwnedMemberMatch partialMatch) {
    return rawAccumulateAllValuesOfsource(partialMatch.toArray());
  }
  
  /**
   * Retrieve the set of values that occur in matches for source.
   * @return the Set of all values, null if no parameter with the given name exists, empty set if there are no matches
   * 
   */
  public Set<Namespace> getAllValuesOfsource(final NamedElement pTarget) {
    return rawAccumulateAllValuesOfsource(new Object[]{
    null, 
    pTarget
    });
  }
  
  /**
   * Retrieve the set of values that occur in matches for target.
   * @return the Set of all values, null if no parameter with the given name exists, empty set if there are no matches
   * 
   */
  protected Set<NamedElement> rawAccumulateAllValuesOftarget(final Object[] parameters) {
    Set<NamedElement> results = new HashSet<NamedElement>();
    rawAccumulateAllValues(POSITION_TARGET, parameters, results);
    return results;
  }
  
  /**
   * Retrieve the set of values that occur in matches for target.
   * @return the Set of all values, null if no parameter with the given name exists, empty set if there are no matches
   * 
   */
  public Set<NamedElement> getAllValuesOftarget() {
    return rawAccumulateAllValuesOftarget(emptyArray());
  }
  
  /**
   * Retrieve the set of values that occur in matches for target.
   * @return the Set of all values, null if no parameter with the given name exists, empty set if there are no matches
   * 
   */
  public Set<NamedElement> getAllValuesOftarget(final NamespaceOwnedMemberMatch partialMatch) {
    return rawAccumulateAllValuesOftarget(partialMatch.toArray());
  }
  
  /**
   * Retrieve the set of values that occur in matches for target.
   * @return the Set of all values, null if no parameter with the given name exists, empty set if there are no matches
   * 
   */
  public Set<NamedElement> getAllValuesOftarget(final Namespace pSource) {
    return rawAccumulateAllValuesOftarget(new Object[]{
    pSource, 
    null
    });
  }
  
  @Override
  protected NamespaceOwnedMemberMatch tupleToMatch(final Tuple t) {
    try {
    	return NamespaceOwnedMemberMatch.newMatch((org.eclipse.uml2.uml.Namespace) t.get(POSITION_SOURCE), (org.eclipse.uml2.uml.NamedElement) t.get(POSITION_TARGET));
    } catch(ClassCastException e) {
    	LOGGER.error("Element(s) in tuple not properly typed!",e);
    	return null;
    }
  }
  
  @Override
  protected NamespaceOwnedMemberMatch arrayToMatch(final Object[] match) {
    try {
    	return NamespaceOwnedMemberMatch.newMatch((org.eclipse.uml2.uml.Namespace) match[POSITION_SOURCE], (org.eclipse.uml2.uml.NamedElement) match[POSITION_TARGET]);
    } catch(ClassCastException e) {
    	LOGGER.error("Element(s) in array not properly typed!",e);
    	return null;
    }
  }
  
  @Override
  protected NamespaceOwnedMemberMatch arrayToMatchMutable(final Object[] match) {
    try {
    	return NamespaceOwnedMemberMatch.newMutableMatch((org.eclipse.uml2.uml.Namespace) match[POSITION_SOURCE], (org.eclipse.uml2.uml.NamedElement) match[POSITION_TARGET]);
    } catch(ClassCastException e) {
    	LOGGER.error("Element(s) in array not properly typed!",e);
    	return null;
    }
  }
  
  /**
   * @return the singleton instance of the query specification of this pattern
   * @throws IncQueryException if the pattern definition could not be loaded
   * 
   */
  public static IQuerySpecification<NamespaceOwnedMemberMatcher> querySpecification() throws IncQueryException {
    return NamespaceOwnedMemberQuerySpecification.instance();
  }
}
