package org.eclipse.incquery.uml.derivedfeatures;

import java.util.Collection;
import java.util.HashSet;
import java.util.Set;
import org.apache.log4j.Logger;
import org.eclipse.emf.common.notify.Notifier;
import org.eclipse.incquery.runtime.api.IMatchProcessor;
import org.eclipse.incquery.runtime.api.IQuerySpecification;
import org.eclipse.incquery.runtime.api.IncQueryEngine;
import org.eclipse.incquery.runtime.api.impl.BaseMatcher;
import org.eclipse.incquery.runtime.exception.IncQueryException;
import org.eclipse.incquery.runtime.matchers.tuple.Tuple;
import org.eclipse.incquery.runtime.util.IncQueryLoggingUtil;
import org.eclipse.incquery.uml.derivedfeatures.ProtocolTransitionReferredMatch;
import org.eclipse.incquery.uml.derivedfeatures.util.ProtocolTransitionReferredQuerySpecification;
import org.eclipse.uml2.uml.Operation;
import org.eclipse.uml2.uml.ProtocolTransition;

/**
 * Generated pattern matcher API of the org.eclipse.incquery.uml.derivedfeatures.protocolTransitionReferred pattern,
 * providing pattern-specific query methods.
 * 
 * <p>Use the pattern matcher on a given model via {@link #on(IncQueryEngine)},
 * e.g. in conjunction with {@link IncQueryEngine#on(Notifier)}.
 * 
 * <p>Matches of the pattern will be represented as {@link ProtocolTransitionReferredMatch}.
 * 
 * <p>Original source:
 * <code><pre>
 * (if self.name {@literal <}{@literal >} null and self.allNamespaces()-{@literal >}select( ns | ns.name=null )-{@literal >}isEmpty()
 * then 
 *     self.allNamespaces()-{@literal >}iterate( ns : Namespace; agg: String = self.name | ns.name.concat(self.separator()).concat(agg))
 * else
 *    null
 * endif)
 * 
 * // Can't compile OCL to IncQuery because of java.lang.IllegalArgumentException: Unhandled parameter types: [self.allNamespaces()-{@literal >}iterate(ns : Namespace; agg : String = self.name | ns.name.concat(self.separator()).concat(agg))]
 * //{@literal @}Surrogate(feature = "qualifiedName")
 * //pattern namedElementQualifiedName(source: NamedElement, target: String) {}
 * 
 * 
 * (self.importMembers(elementImport.importedElement-{@literal >}asSet()-{@literal >}union(packageImport.importedPackage-{@literal >}collect(p | p.visibleMembers()))-{@literal >}asSet()))
 * 
 * // Can't compile OCL to IncQuery because of java.lang.UnsupportedOperationException: visibleMembers
 * //{@literal @}Surrogate(feature = "importedMember")
 * //pattern namespaceImportedMember(source: Namespace, target: PackageableElement) {}
 * 
 * 
 * (if behavior = null then
 *     null
 * else
 *     behavior.ownedParameter-{@literal >}first()
 * endif)
 * 
 * // Can't compile OCL to IncQuery because of java.lang.IllegalArgumentException: Unhandled parameter types: [null]
 * //{@literal @}Surrogate(feature = "result")
 * //pattern opaqueExpressionResult(source: OpaqueExpression, target: Parameter) {}
 * 
 * 
 * (if returnResult()-{@literal >}notEmpty() then returnResult()-{@literal >} exists(isOrdered) else false endif)
 * 
 * // Can't compile OCL to IncQuery because of java.lang.UnsupportedOperationException: returnResult
 * //{@literal @}Surrogate(feature = "isOrdered")
 * //pattern operationIsOrdered(source: Operation, target: Boolean) {}
 * 
 * 
 * (if returnResult()-{@literal >}notEmpty() then returnResult()-{@literal >}exists(isUnique) else true endif)
 * 
 * // Can't compile OCL to IncQuery because of java.lang.UnsupportedOperationException: returnResult
 * //{@literal @}Surrogate(feature = "isUnique")
 * //pattern operationIsUnique(source: Operation, target: Boolean) {}
 * 
 * 
 * (if returnResult()-{@literal >}notEmpty() then returnResult()-{@literal >}any(true).lower else null endif)
 * 
 * // Can't compile OCL to IncQuery because of java.lang.UnsupportedOperationException: returnResult
 * //{@literal @}Surrogate(feature = "lower")
 * //pattern operationLower(source: Operation, target: Integer) {}
 * 
 * 
 * (if returnResult()-{@literal >}notEmpty() then returnResult()-{@literal >}any(true).type else null endif)
 * 
 * // Can't compile OCL to IncQuery because of java.lang.UnsupportedOperationException: returnResult
 * //{@literal @}Surrogate(feature = "type")
 * //pattern operationType(source: Operation, target: Type) {}
 * 
 * 
 * (if returnResult()-{@literal >}notEmpty() then returnResult()-{@literal >}any(true).upper else null endif)
 * 
 * // Can't compile OCL to IncQuery because of java.lang.UnsupportedOperationException: returnResult
 * //{@literal @}Surrogate(feature = "upper")
 * //pattern operationUpper(source: Operation, target: UnlimitedNatural) {}
 * 
 * 
 * (if self.type = String then defaultValue.stringValue() else null endif)
 * 
 * // Can't compile OCL to IncQuery because of java.lang.UnsupportedOperationException: stringValue
 * //{@literal @}Surrogate(feature = "default")
 * //pattern parameterDefault(source: Parameter, target: String) {}
 * 
 * 
 * (if isConjugated then basicRequired() else basicProvided() endif)
 * 
 * // Can't compile OCL to IncQuery because of java.lang.UnsupportedOperationException: basicRequired
 * //{@literal @}Surrogate(feature = "provided")
 * //pattern portProvided(source: Port, target: Interface) {}
 * 
 * 
 * (if isConjugated then basicProvided() else basicRequired() endif)
 * 
 * // Can't compile OCL to IncQuery because of java.lang.UnsupportedOperationException: basicProvided
 * //{@literal @}Surrogate(feature = "required")
 * //pattern portRequired(source: Port, target: Interface) {}
 * 
 * 
 * (if association {@literal <}{@literal >} null and association.memberEnd-{@literal >}size() = 2
 * then
 *     association.memberEnd-{@literal >}any(e | e {@literal <}{@literal >} self)
 * else
 *     null
 * endif)
 * 
 * // Can't compile OCL to IncQuery because of java.lang.IllegalArgumentException: Iterator expression kind must be collect, select or reject instead of any in self.association.memberEnd-{@literal >}any(e : Property | e.{@literal <}{@literal >}(self))
 * //{@literal @}Surrogate(feature = "opposite")
 * //pattern propertyOpposite(source: Property, target: Property) {}
 * 
 * 
 * (trigger-{@literal >}collect(event)-{@literal >}select(oclIsKindOf(CallEvent))-{@literal >}collect(oclAsType(CallEvent).operation)-{@literal >}asSet())
 * 
 * {@literal @}Surrogate(feature = "referred")
 * {@literal @}QueryExplorer(checked = false)
 * pattern protocolTransitionReferred(
 *     self : ProtocolTransition,
 *     operation : Operation
 * ) {
 *     CallEvent(callEvent);
 *     temp2 == callEvent;
 *     Transition.trigger(self, trigger);
 *     temp1 == trigger;
 *     Trigger.event(temp1, event);
 *     temp2 == event;
 *     temp3 == temp2;
 *     CallEvent.operation(temp3, operation);
 * }
 * </pre></code>
 * 
 * @see ProtocolTransitionReferredMatch
 * @see ProtocolTransitionReferredProcessor
 * @see ProtocolTransitionReferredQuerySpecification
 * 
 */
@SuppressWarnings("all")
public class ProtocolTransitionReferredMatcher extends BaseMatcher<ProtocolTransitionReferredMatch> {
  /**
   * Initializes the pattern matcher within an existing EMF-IncQuery engine.
   * If the pattern matcher is already constructed in the engine, only a light-weight reference is returned.
   * The match set will be incrementally refreshed upon updates.
   * @param engine the existing EMF-IncQuery engine in which this matcher will be created.
   * @throws IncQueryException if an error occurs during pattern matcher creation
   * 
   */
  public static ProtocolTransitionReferredMatcher on(final IncQueryEngine engine) throws IncQueryException {
    // check if matcher already exists
    ProtocolTransitionReferredMatcher matcher = engine.getExistingMatcher(querySpecification());
    if (matcher == null) {
    	matcher = new ProtocolTransitionReferredMatcher(engine);
    	// do not have to "put" it into engine.matchers, reportMatcherInitialized() will take care of it
    }
    return matcher;
  }
  
  private final static int POSITION_SELF = 0;
  
  private final static int POSITION_OPERATION = 1;
  
  private final static Logger LOGGER = IncQueryLoggingUtil.getLogger(ProtocolTransitionReferredMatcher.class);
  
  /**
   * Initializes the pattern matcher over a given EMF model root (recommended: Resource or ResourceSet).
   * If a pattern matcher is already constructed with the same root, only a light-weight reference is returned.
   * The scope of pattern matching will be the given EMF model root and below (see FAQ for more precise definition).
   * The match set will be incrementally refreshed upon updates from this scope.
   * <p>The matcher will be created within the managed {@link IncQueryEngine} belonging to the EMF model root, so
   * multiple matchers will reuse the same engine and benefit from increased performance and reduced memory footprint.
   * @param emfRoot the root of the EMF containment hierarchy where the pattern matcher will operate. Recommended: Resource or ResourceSet.
   * @throws IncQueryException if an error occurs during pattern matcher creation
   * @deprecated use {@link #on(IncQueryEngine)} instead, e.g. in conjunction with {@link IncQueryEngine#on(Notifier)}
   * 
   */
  @Deprecated
  public ProtocolTransitionReferredMatcher(final Notifier emfRoot) throws IncQueryException {
    this(IncQueryEngine.on(emfRoot));
  }
  
  /**
   * Initializes the pattern matcher within an existing EMF-IncQuery engine.
   * If the pattern matcher is already constructed in the engine, only a light-weight reference is returned.
   * The match set will be incrementally refreshed upon updates.
   * @param engine the existing EMF-IncQuery engine in which this matcher will be created.
   * @throws IncQueryException if an error occurs during pattern matcher creation
   * @deprecated use {@link #on(IncQueryEngine)} instead
   * 
   */
  @Deprecated
  public ProtocolTransitionReferredMatcher(final IncQueryEngine engine) throws IncQueryException {
    super(engine, querySpecification());
  }
  
  /**
   * Returns the set of all matches of the pattern that conform to the given fixed values of some parameters.
   * @param pSelf the fixed value of pattern parameter self, or null if not bound.
   * @param pOperation the fixed value of pattern parameter operation, or null if not bound.
   * @return matches represented as a ProtocolTransitionReferredMatch object.
   * 
   */
  public Collection<ProtocolTransitionReferredMatch> getAllMatches(final ProtocolTransition pSelf, final Operation pOperation) {
    return rawGetAllMatches(new Object[]{pSelf, pOperation});
  }
  
  /**
   * Returns an arbitrarily chosen match of the pattern that conforms to the given fixed values of some parameters.
   * Neither determinism nor randomness of selection is guaranteed.
   * @param pSelf the fixed value of pattern parameter self, or null if not bound.
   * @param pOperation the fixed value of pattern parameter operation, or null if not bound.
   * @return a match represented as a ProtocolTransitionReferredMatch object, or null if no match is found.
   * 
   */
  public ProtocolTransitionReferredMatch getOneArbitraryMatch(final ProtocolTransition pSelf, final Operation pOperation) {
    return rawGetOneArbitraryMatch(new Object[]{pSelf, pOperation});
  }
  
  /**
   * Indicates whether the given combination of specified pattern parameters constitute a valid pattern match,
   * under any possible substitution of the unspecified parameters (if any).
   * @param pSelf the fixed value of pattern parameter self, or null if not bound.
   * @param pOperation the fixed value of pattern parameter operation, or null if not bound.
   * @return true if the input is a valid (partial) match of the pattern.
   * 
   */
  public boolean hasMatch(final ProtocolTransition pSelf, final Operation pOperation) {
    return rawHasMatch(new Object[]{pSelf, pOperation});
  }
  
  /**
   * Returns the number of all matches of the pattern that conform to the given fixed values of some parameters.
   * @param pSelf the fixed value of pattern parameter self, or null if not bound.
   * @param pOperation the fixed value of pattern parameter operation, or null if not bound.
   * @return the number of pattern matches found.
   * 
   */
  public int countMatches(final ProtocolTransition pSelf, final Operation pOperation) {
    return rawCountMatches(new Object[]{pSelf, pOperation});
  }
  
  /**
   * Executes the given processor on each match of the pattern that conforms to the given fixed values of some parameters.
   * @param pSelf the fixed value of pattern parameter self, or null if not bound.
   * @param pOperation the fixed value of pattern parameter operation, or null if not bound.
   * @param processor the action that will process each pattern match.
   * 
   */
  public void forEachMatch(final ProtocolTransition pSelf, final Operation pOperation, final IMatchProcessor<? super ProtocolTransitionReferredMatch> processor) {
    rawForEachMatch(new Object[]{pSelf, pOperation}, processor);
  }
  
  /**
   * Executes the given processor on an arbitrarily chosen match of the pattern that conforms to the given fixed values of some parameters.
   * Neither determinism nor randomness of selection is guaranteed.
   * @param pSelf the fixed value of pattern parameter self, or null if not bound.
   * @param pOperation the fixed value of pattern parameter operation, or null if not bound.
   * @param processor the action that will process the selected match.
   * @return true if the pattern has at least one match with the given parameter values, false if the processor was not invoked
   * 
   */
  public boolean forOneArbitraryMatch(final ProtocolTransition pSelf, final Operation pOperation, final IMatchProcessor<? super ProtocolTransitionReferredMatch> processor) {
    return rawForOneArbitraryMatch(new Object[]{pSelf, pOperation}, processor);
  }
  
  /**
   * Returns a new (partial) match.
   * This can be used e.g. to call the matcher with a partial match.
   * <p>The returned match will be immutable. Use {@link #newEmptyMatch()} to obtain a mutable match object.
   * @param pSelf the fixed value of pattern parameter self, or null if not bound.
   * @param pOperation the fixed value of pattern parameter operation, or null if not bound.
   * @return the (partial) match object.
   * 
   */
  public ProtocolTransitionReferredMatch newMatch(final ProtocolTransition pSelf, final Operation pOperation) {
    return ProtocolTransitionReferredMatch.newMatch(pSelf, pOperation);
  }
  
  /**
   * Retrieve the set of values that occur in matches for self.
   * @return the Set of all values, null if no parameter with the given name exists, empty set if there are no matches
   * 
   */
  protected Set<ProtocolTransition> rawAccumulateAllValuesOfself(final Object[] parameters) {
    Set<ProtocolTransition> results = new HashSet<ProtocolTransition>();
    rawAccumulateAllValues(POSITION_SELF, parameters, results);
    return results;
  }
  
  /**
   * Retrieve the set of values that occur in matches for self.
   * @return the Set of all values, null if no parameter with the given name exists, empty set if there are no matches
   * 
   */
  public Set<ProtocolTransition> getAllValuesOfself() {
    return rawAccumulateAllValuesOfself(emptyArray());
  }
  
  /**
   * Retrieve the set of values that occur in matches for self.
   * @return the Set of all values, null if no parameter with the given name exists, empty set if there are no matches
   * 
   */
  public Set<ProtocolTransition> getAllValuesOfself(final ProtocolTransitionReferredMatch partialMatch) {
    return rawAccumulateAllValuesOfself(partialMatch.toArray());
  }
  
  /**
   * Retrieve the set of values that occur in matches for self.
   * @return the Set of all values, null if no parameter with the given name exists, empty set if there are no matches
   * 
   */
  public Set<ProtocolTransition> getAllValuesOfself(final Operation pOperation) {
    return rawAccumulateAllValuesOfself(new Object[]{
    null, 
    pOperation
    });
  }
  
  /**
   * Retrieve the set of values that occur in matches for operation.
   * @return the Set of all values, null if no parameter with the given name exists, empty set if there are no matches
   * 
   */
  protected Set<Operation> rawAccumulateAllValuesOfoperation(final Object[] parameters) {
    Set<Operation> results = new HashSet<Operation>();
    rawAccumulateAllValues(POSITION_OPERATION, parameters, results);
    return results;
  }
  
  /**
   * Retrieve the set of values that occur in matches for operation.
   * @return the Set of all values, null if no parameter with the given name exists, empty set if there are no matches
   * 
   */
  public Set<Operation> getAllValuesOfoperation() {
    return rawAccumulateAllValuesOfoperation(emptyArray());
  }
  
  /**
   * Retrieve the set of values that occur in matches for operation.
   * @return the Set of all values, null if no parameter with the given name exists, empty set if there are no matches
   * 
   */
  public Set<Operation> getAllValuesOfoperation(final ProtocolTransitionReferredMatch partialMatch) {
    return rawAccumulateAllValuesOfoperation(partialMatch.toArray());
  }
  
  /**
   * Retrieve the set of values that occur in matches for operation.
   * @return the Set of all values, null if no parameter with the given name exists, empty set if there are no matches
   * 
   */
  public Set<Operation> getAllValuesOfoperation(final ProtocolTransition pSelf) {
    return rawAccumulateAllValuesOfoperation(new Object[]{
    pSelf, 
    null
    });
  }
  
  @Override
  protected ProtocolTransitionReferredMatch tupleToMatch(final Tuple t) {
    try {
    	return ProtocolTransitionReferredMatch.newMatch((org.eclipse.uml2.uml.ProtocolTransition) t.get(POSITION_SELF), (org.eclipse.uml2.uml.Operation) t.get(POSITION_OPERATION));
    } catch(ClassCastException e) {
    	LOGGER.error("Element(s) in tuple not properly typed!",e);
    	return null;
    }
  }
  
  @Override
  protected ProtocolTransitionReferredMatch arrayToMatch(final Object[] match) {
    try {
    	return ProtocolTransitionReferredMatch.newMatch((org.eclipse.uml2.uml.ProtocolTransition) match[POSITION_SELF], (org.eclipse.uml2.uml.Operation) match[POSITION_OPERATION]);
    } catch(ClassCastException e) {
    	LOGGER.error("Element(s) in array not properly typed!",e);
    	return null;
    }
  }
  
  @Override
  protected ProtocolTransitionReferredMatch arrayToMatchMutable(final Object[] match) {
    try {
    	return ProtocolTransitionReferredMatch.newMutableMatch((org.eclipse.uml2.uml.ProtocolTransition) match[POSITION_SELF], (org.eclipse.uml2.uml.Operation) match[POSITION_OPERATION]);
    } catch(ClassCastException e) {
    	LOGGER.error("Element(s) in array not properly typed!",e);
    	return null;
    }
  }
  
  /**
   * @return the singleton instance of the query specification of this pattern
   * @throws IncQueryException if the pattern definition could not be loaded
   * 
   */
  public static IQuerySpecification<ProtocolTransitionReferredMatcher> querySpecification() throws IncQueryException {
    return ProtocolTransitionReferredQuerySpecification.instance();
  }
}
