package org.eclipse.incquery.uml.derivedfeatures.util;

import com.google.common.collect.Sets;
import java.util.Arrays;
import java.util.List;
import java.util.Set;
import org.eclipse.emf.ecore.EClass;
import org.eclipse.incquery.runtime.api.IncQueryEngine;
import org.eclipse.incquery.runtime.api.impl.BaseGeneratedEMFPQuery;
import org.eclipse.incquery.runtime.api.impl.BaseGeneratedEMFQuerySpecification;
import org.eclipse.incquery.runtime.emf.types.EClassTransitiveInstancesKey;
import org.eclipse.incquery.runtime.emf.types.EStructuralFeatureInstancesKey;
import org.eclipse.incquery.runtime.exception.IncQueryException;
import org.eclipse.incquery.runtime.matchers.psystem.PBody;
import org.eclipse.incquery.runtime.matchers.psystem.PVariable;
import org.eclipse.incquery.runtime.matchers.psystem.annotations.PAnnotation;
import org.eclipse.incquery.runtime.matchers.psystem.basicdeferred.Equality;
import org.eclipse.incquery.runtime.matchers.psystem.basicdeferred.ExportedParameter;
import org.eclipse.incquery.runtime.matchers.psystem.basicdeferred.Inequality;
import org.eclipse.incquery.runtime.matchers.psystem.basicenumerables.ConstantValue;
import org.eclipse.incquery.runtime.matchers.psystem.basicenumerables.TypeConstraint;
import org.eclipse.incquery.runtime.matchers.psystem.queries.PParameter;
import org.eclipse.incquery.runtime.matchers.psystem.queries.QueryInitializationException;
import org.eclipse.incquery.runtime.matchers.tuple.FlatTuple;
import org.eclipse.incquery.uml.derivedfeatures.PropertyIsCompositeMatch;
import org.eclipse.incquery.uml.derivedfeatures.PropertyIsCompositeMatcher;

/**
 * A pattern-specific query specification that can instantiate PropertyIsCompositeMatcher in a type-safe way.
 * 
 * @see PropertyIsCompositeMatcher
 * @see PropertyIsCompositeMatch
 * 
 */
@SuppressWarnings("all")
public final class PropertyIsCompositeQuerySpecification extends BaseGeneratedEMFQuerySpecification<PropertyIsCompositeMatcher> {
  private PropertyIsCompositeQuerySpecification() {
    super(GeneratedPQuery.INSTANCE);
  }
  
  /**
   * @return the singleton instance of the query specification
   * @throws IncQueryException if the pattern definition could not be loaded
   * 
   */
  public static PropertyIsCompositeQuerySpecification instance() throws IncQueryException {
    try{
    	return LazyHolder.INSTANCE;
    } catch (ExceptionInInitializerError err) {
    	throw processInitializerError(err);
    }
  }
  
  @Override
  protected PropertyIsCompositeMatcher instantiate(final IncQueryEngine engine) throws IncQueryException {
    return PropertyIsCompositeMatcher.on(engine);
  }
  
  @Override
  public PropertyIsCompositeMatch newEmptyMatch() {
    return PropertyIsCompositeMatch.newEmptyMatch();
  }
  
  @Override
  public PropertyIsCompositeMatch newMatch(final Object... parameters) {
    return PropertyIsCompositeMatch.newMatch((org.eclipse.uml2.uml.Property) parameters[0], (java.lang.Boolean) parameters[1]);
  }
  
  private static class LazyHolder {
    private final static PropertyIsCompositeQuerySpecification INSTANCE = make();
    
    public static PropertyIsCompositeQuerySpecification make() {
      return new PropertyIsCompositeQuerySpecification();					
    }
  }
  
  private static class GeneratedPQuery extends BaseGeneratedEMFPQuery {
    private final static PropertyIsCompositeQuerySpecification.GeneratedPQuery INSTANCE = new GeneratedPQuery();
    
    @Override
    public String getFullyQualifiedName() {
      return "org.eclipse.incquery.uml.derivedfeatures.propertyIsComposite";
    }
    
    @Override
    public List<String> getParameterNames() {
      return Arrays.asList("property","isComposite");
    }
    
    @Override
    public List<PParameter> getParameters() {
      return Arrays.asList(new PParameter("property", "org.eclipse.uml2.uml.Property"),new PParameter("isComposite", "java.lang.Boolean"));
    }
    
    @Override
    public Set<PBody> doGetContainedBodies() throws QueryInitializationException {
      Set<PBody> bodies = Sets.newLinkedHashSet();
      try {
      {
      	PBody body = new PBody(this);
      	PVariable var_property = body.getOrCreateVariableByName("property");
      	PVariable var_isComposite = body.getOrCreateVariableByName("isComposite");
      	PVariable var__virtual_0_ = body.getOrCreateVariableByName(".virtual{0}");
      	PVariable var__virtual_1_ = body.getOrCreateVariableByName(".virtual{1}");
      	PVariable var__virtual_2_ = body.getOrCreateVariableByName(".virtual{2}");
      	body.setExportedParameters(Arrays.<ExportedParameter>asList(
      		new ExportedParameter(body, var_property, "property"),
      				
      		new ExportedParameter(body, var_isComposite, "isComposite")
      	));
      	new TypeConstraint(body, new FlatTuple(var_property), new EClassTransitiveInstancesKey((EClass)getClassifierLiteral("http://www.eclipse.org/uml2/5.0.0/UML", "Property")));
      	new ConstantValue(body, var__virtual_0_, org.eclipse.uml2.uml.AggregationKind.get("composite"));
      	new TypeConstraint(body, new FlatTuple(var_property), new EClassTransitiveInstancesKey((EClass)getClassifierLiteral("http://www.eclipse.org/uml2/5.0.0/UML", "Property")));
      	new TypeConstraint(body, new FlatTuple(var_property, var__virtual_1_), new EStructuralFeatureInstancesKey(getFeatureLiteral("http://www.eclipse.org/uml2/5.0.0/UML", "Property", "aggregation")));
      	new Equality(body, var__virtual_1_, var__virtual_0_);
      	new ConstantValue(body, var__virtual_2_, true);
      	new Equality(body, var_isComposite, var__virtual_2_);
      	bodies.add(body);
      }
      {
      	PBody body = new PBody(this);
      	PVariable var_property = body.getOrCreateVariableByName("property");
      	PVariable var_isComposite = body.getOrCreateVariableByName("isComposite");
      	PVariable var_aggregation = body.getOrCreateVariableByName("aggregation");
      	PVariable var__virtual_0_ = body.getOrCreateVariableByName(".virtual{0}");
      	PVariable var__virtual_1_ = body.getOrCreateVariableByName(".virtual{1}");
      	PVariable var__virtual_2_ = body.getOrCreateVariableByName(".virtual{2}");
      	body.setExportedParameters(Arrays.<ExportedParameter>asList(
      		new ExportedParameter(body, var_property, "property"),
      				
      		new ExportedParameter(body, var_isComposite, "isComposite")
      	));
      	new TypeConstraint(body, new FlatTuple(var_property), new EClassTransitiveInstancesKey((EClass)getClassifierLiteral("http://www.eclipse.org/uml2/5.0.0/UML", "Property")));
      	new TypeConstraint(body, new FlatTuple(var_property), new EClassTransitiveInstancesKey((EClass)getClassifierLiteral("http://www.eclipse.org/uml2/5.0.0/UML", "Property")));
      	new TypeConstraint(body, new FlatTuple(var_property, var__virtual_0_), new EStructuralFeatureInstancesKey(getFeatureLiteral("http://www.eclipse.org/uml2/5.0.0/UML", "Property", "aggregation")));
      	new Equality(body, var__virtual_0_, var_aggregation);
      	new ConstantValue(body, var__virtual_1_, org.eclipse.uml2.uml.AggregationKind.get("composite"));
      	new Inequality(body, var_aggregation, var__virtual_1_);
      	new ConstantValue(body, var__virtual_2_, false);
      	new Equality(body, var_isComposite, var__virtual_2_);
      	bodies.add(body);
      }
      	{
      	PAnnotation annotation = new PAnnotation("Surrogate");
      	annotation.addAttribute("feature", "isComposite");
      	addAnnotation(annotation);
      }
      {
      	PAnnotation annotation = new PAnnotation("QueryExplorer");
      	annotation.addAttribute("checked", false);
      	addAnnotation(annotation);
      }
      	// to silence compiler error
      	if (false) throw new IncQueryException("Never", "happens");
      } catch (IncQueryException ex) {
      	throw processDependencyException(ex);
      }
      return bodies;
    }
  }
}
